@@ -42,7 +42,7 @@ enum
 	EPHY_NODE_BMK_PROP_LOCATION = 3,
 	EPHY_NODE_BMK_PROP_KEYWORDS = 4,
 	EPHY_NODE_KEYWORD_PROP_NAME = 5,
-	EPHY_NODE_BMK_PROP_SMART_LOCATION = 6,
+	EPHY_NODE_BMK_PROP_HAS_SMART_ADDRESS = 9,
 	EPHY_NODE_BMK_PROP_ICON = 7,
 	EPHY_NODE_KEYWORD_PROP_PRIORITY = 8
 };
@@ -73,8 +73,7 @@ void		 ephy_bookmarks_save		(EphyBookmarks *eb);
 
 EphyNode        *ephy_bookmarks_add		(EphyBookmarks *eb,
 						 const char *title,
-						 const char *url,
-						 const char *smart_url);
+						 const char *url);
 
 EphyNode*	 ephy_bookmarks_find_bookmark   (EphyBookmarks *eb,
 						 const char *url);
@@ -83,6 +82,10 @@ void		 ephy_bookmarks_set_icon	(EphyBookmarks *eb,
 						 const char *url,
 						 const char *icon);
 
+void		 ephy_bookmarks_set_address     (EphyBookmarks *eb,
+			                         EphyNode *bookmark,
+			                         const char *address);
+
 char		*ephy_bookmarks_solve_smart_url (EphyBookmarks *eb,
 						 const char *smart_url,
 						 const char *content);
