@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-window.c,v 1.90 2003/06/23 09:57:21 mpeseng Exp $
+ *  $Id: ephy-window.c,v 1.92 2003/06/26 14:31:07 mpeseng Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -1304,7 +1304,6 @@ tab_added_cb (EphyNotebook *notebook, GtkWidget *child, EphyWindow *window)
 	sync_tab_load_status (tab, NULL, window);
 	g_signal_connect_object (G_OBJECT (tab), "notify::load-status",
 				 G_CALLBACK (sync_tab_load_status), window, 0);
-	sync_tab_visibility (tab, NULL, window);
 	g_signal_connect_object (G_OBJECT (tab), "notify::visible",
 				 G_CALLBACK (sync_tab_visibility), window, 0);
 }
@@ -1888,22 +1887,21 @@ ephy_window_print (EphyWindow *window)
 	EphyDialog *dialog;
 	EphyEmbed *embed;
 
-	if (window->priv->print_dialog)
-	{
-		dialog = window->priv->print_dialog;
-	}
-	else
+	if (window->priv->print_dialog == NULL)
 	{
 		embed = ephy_window_get_active_embed (window);
-		dialog = print_dialog_new_with_parent (GTK_WIDGET(window),
-						       embed, NULL);
-		g_signal_connect (G_OBJECT(dialog),
+		dialog = print_dialog_new_with_parent
+					(GTK_WIDGET(window), embed, NULL);
+		g_signal_connect (G_OBJECT (dialog),
 				  "preview",
 				  G_CALLBACK (print_dialog_preview_cb),
 				  window);
+		window->priv->print_dialog = dialog;
+		g_object_add_weak_pointer(G_OBJECT (dialog),
+					  (gpointer *) &window->priv->print_dialog);
 	}
 
-	ephy_dialog_show (EPHY_DIALOG (dialog));
+	ephy_dialog_show (window->priv->print_dialog);
 }
 
 void
