@@ -55,6 +55,8 @@ pdm_dialog_cookies_treeview_selection_changed_cb (GtkTreeSelection *selection,
 void
 pdm_dialog_passwords_treeview_selection_changed_cb (GtkTreeSelection *selection,
 						    PdmDialog *dialog);
+void
+pdm_dialog_response_cb (GtkDialog *dialog, gint response_id, gpointer data);
 
 static GObjectClass *parent_class = NULL;
 
@@ -79,6 +81,8 @@ struct PdmDialogPrivate
 
 enum
 {
+	PROP_WINDOW,
+	PROP_NOTEBOOK,
 	PROP_COOKIES_TREEVIEW,
 	PROP_COOKIES_REMOVE,
 	PROP_PASSWORDS_TREEVIEW,
@@ -104,6 +108,9 @@ enum
 static const
 EphyDialogProperty properties [] =
 {
+	{ PROP_WINDOW, "pdm_dialog", NULL, PT_NORMAL, NULL },
+	{ PROP_NOTEBOOK, "pdm_notebook", NULL, PT_NORMAL, NULL },
+
 	{ PROP_COOKIES_TREEVIEW, "cookies_treeview", NULL, PT_NORMAL, NULL },
 	{ PROP_COOKIES_REMOVE, "cookies_remove_button", NULL, PT_NORMAL, NULL },
 	{ PROP_PASSWORDS_TREEVIEW, "passwords_treeview", NULL, PT_NORMAL, NULL },
@@ -153,26 +160,44 @@ pdm_dialog_class_init (PdmDialogClass *klass)
         object_class->finalize = pdm_dialog_finalize;
 }
 
+static void
+pdm_dialog_show_help (PdmDialog *pd)
+{
+	GtkWidget *notebook, *window;
+	gint id;
+
+	/* FIXME: Once we actually have documentation we
+	 * should point these at the correct links.
+	 */
+	gchar *help_preferences[] = {
+		"pdm",
+		"pdm"
+	};
+
+	window = ephy_dialog_get_control (EPHY_DIALOG (pd), PROP_WINDOW);
+	g_return_if_fail (GTK_IS_WINDOW (window));
+
+	notebook = ephy_dialog_get_control (EPHY_DIALOG (pd), PROP_NOTEBOOK);
+	g_return_if_fail (notebook != NULL);
+
+	id = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
+	g_assert (id == 0 || id == 1);
+
+	ephy_gui_help (GTK_WINDOW (window), "epiphany", help_preferences[id]);
+}
+
 static void
 cookies_treeview_selection_changed_cb (GtkTreeSelection *selection,
                                        PdmDialog *dialog)
 {
 	GtkWidget *widget;
-	GList *l;
 	EphyDialog *d = EPHY_DIALOG(dialog);
 	gboolean has_selection;
-	GtkTreeModel *model;
 
-	l = gtk_tree_selection_get_selected_rows
-		(selection, &model);
-
-	has_selection = l != NULL;
+	has_selection = gtk_tree_selection_count_selected_rows (selection) == 1;
 
 	widget = ephy_dialog_get_control (d, PROP_COOKIES_PROPERTIES);
 	gtk_widget_set_sensitive (widget, has_selection);
-
-	g_list_foreach (l, (GFunc)gtk_tree_path_free, NULL);
-	g_list_free (l);
 }
 
 static void
@@ -180,21 +205,13 @@ action_treeview_selection_changed_cb (GtkTreeSelection *selection,
                                       PdmActionInfo *action)
 {
 	GtkWidget *widget;
-	GList *l;
 	EphyDialog *d = EPHY_DIALOG(action->dialog);
 	gboolean has_selection;
-	GtkTreeModel *model;
 
-	l = gtk_tree_selection_get_selected_rows
-		(selection, &model);
-
-	has_selection = l != NULL;
+	has_selection = gtk_tree_selection_count_selected_rows (selection) > 0;
 
 	widget = ephy_dialog_get_control (d, action->remove_id);
 	gtk_widget_set_sensitive (widget, has_selection);
-
-	g_list_foreach (l, (GFunc)gtk_tree_path_free, NULL);
-	g_list_free (l);
 }
 
 static GtkWidget *
@@ -312,22 +329,21 @@ static void
 pdm_dialog_remove_button_clicked_cb (GtkWidget *button,
 				     PdmActionInfo *action)
 {
-	GList *l, *r = NULL;
+	GList *llist, *rlist = NULL, *l, *r;
 	GList *remove_list = NULL;
 	GtkTreeModel *model;
 	GtkTreeSelection *selection;
 
 	selection = gtk_tree_view_get_selection
 		(GTK_TREE_VIEW(action->treeview));
-	l = gtk_tree_selection_get_selected_rows
-		(selection, &model);
-	for (;l != NULL; l = l->next)
+	llist = gtk_tree_selection_get_selected_rows (selection, &model);
+	for (l = llist;l != NULL; l = l->next)
 	{
-		r = g_list_append (r, gtk_tree_row_reference_new
-				   (model, (GtkTreePath *)l->data));
+		rlist = g_list_prepend (rlist, gtk_tree_row_reference_new
+					(model, (GtkTreePath *)l->data));
 	}
 
-	for (; r != NULL; r = r->next)
+	for (r = rlist; r != NULL; r = r->next)
 	{
 		GtkTreeIter iter;
 		gpointer data;
@@ -358,10 +374,9 @@ pdm_dialog_remove_button_clicked_cb (GtkWidget *button,
 		action->free (action, remove_list);
 	}
 
-	l = g_list_first (l);
-	g_list_foreach (l, (GFunc)gtk_tree_path_free, NULL);
-	g_list_free (l);
-	g_list_free (r);
+	g_list_foreach (llist, (GFunc)gtk_tree_path_free, NULL);
+	g_list_free (llist);
+	g_list_free (rlist);
 }
 
 static void
@@ -710,8 +725,16 @@ pdm_dialog_cookies_properties_button_clicked_cb (GtkWidget *button,
 }
 
 void
-pdm_dialog_close_button_clicked_cb (GtkWidget *button,
-			            PdmDialog *dialog)
+pdm_dialog_response_cb (GtkDialog *dialog, gint response_id, gpointer data)
 {
-	g_object_unref (dialog);
+	if (response_id == GTK_RESPONSE_CLOSE)
+	{
+		gtk_widget_destroy (GTK_WIDGET(dialog));
+	}
+	else if (response_id == GTK_RESPONSE_HELP)
+	{
+		g_return_if_fail (IS_PDM_DIALOG (data));
+
+		pdm_dialog_show_help (PDM_DIALOG (data));
+	}
 }
